@@ -201,6 +201,7 @@ class Dashboard:
 
         def write_page(page):
             doc = document(title=page.title)
+            effective_width = page.page_width or self.page_width or 900
             with doc.head:
                 doc.head.add(link(rel="stylesheet", href="../assets/css/style.css"))
                 doc.head.add(script(type="text/javascript", src="../assets/js/script.js"))
@@ -209,6 +210,8 @@ class Dashboard:
                 doc.head.add(script(src="https://cdn.jsdelivr.net/npm/prismjs@1.29.0/prism.min.js"))
                 doc.head.add(script(src="https://cdn.jsdelivr.net/npm/prismjs@1.29.0/components/prism-python.min.js"))
                 doc.head.add(script(src="https://cdn.jsdelivr.net/npm/prismjs@1.29.0/components/prism-javascript.min.js"))
+                # Inject dynamic page width
+                doc.head.add(raw_util(f"<style>.content-inner {{ max-width: {effective_width}px !important; }}</style>"))
             with doc:
                 with div(id="sidebar"):
                     a(self.title, href="../index.html", cls="sidebar-title")
@@ -227,7 +230,9 @@ class Dashboard:
         for page in self.pages:
             write_page(page)
 
+        # Index page
         index_doc = document(title=self.title)
+        effective_width = self.pages[0].page_width or self.page_width or 900
         with index_doc.head:
             index_doc.head.add(link(rel="stylesheet", href="assets/css/style.css"))
             index_doc.head.add(script(type="text/javascript", src="assets/js/script.js"))
@@ -241,7 +246,8 @@ class Dashboard:
             index_doc.head.add(script(src="https://cdn.jsdelivr.net/npm/prismjs@1.29.0/components/prism-bash.min.js"))
             index_doc.head.add(script(src="https://cdn.jsdelivr.net/npm/prismjs@1.29.0/components/prism-json.min.js"))
             index_doc.head.add(script(src="https://cdn.jsdelivr.net/npm/prismjs@1.29.0/components/prism-c.min.js"))
-
+            # Inject dynamic page width
+            index_doc.head.add(raw_util(f"<style>.content-inner {{ max-width: {effective_width}px !important; }}</style>"))
         with index_doc:
             with div(id="sidebar"):
                 a(self.title, href="index.html", cls="sidebar-title")
@@ -260,6 +266,9 @@ class Dashboard:
         from reportlab.lib.pagesizes import A4, letter
         page_size = A4 if pagesize.upper() == "A4" else letter
 
+        import plotly.io as pio
+        pio.kaleido.scope.default_format = "png"
+
         styles = getSampleStyleSheet()
         styles['Heading1'].fontSize = 18
         styles['Heading1'].leading = 22
@@ -277,22 +286,25 @@ class Dashboard:
 
         story = []
         outline_entries = []
+        heading_paragraphs = []
 
         class MyDocTemplate(SimpleDocTemplate):
-            def __init__(self, *args, outline_entries=None, **kwargs):
+            def __init__(self, *args, outline_entries=None, headings=None, **kwargs):
                 super().__init__(*args, **kwargs)
                 self.outline_entries = outline_entries or []
+                self.headings = headings or []
                 self._outline_idx = 0
 
             def afterFlowable(self, flowable):
                 if hasattr(flowable, 'getPlainText'):
-                    text = flowable.getPlainText()
+                    text = flowable.getPlainText().strip()
                     if self._outline_idx < len(self.outline_entries):
-                        title, level, section_num = self.outline_entries[self._outline_idx]
-                        if title in text:
+                        expected_title, level, section_num = self.outline_entries[self._outline_idx]
+                        expected = expected_title.strip()
+                        if text == expected:
                             bookmark_name = f"section_{section_num.replace('.', '_')}"
                             self.canv.bookmarkPage(bookmark_name)
-                            self.canv.addOutlineEntry(title, bookmark_name, level=level, closed=False)
+                            self.canv.addOutlineEntry(expected_title, bookmark_name, level=level, closed=False)
                             self._outline_idx += 1
 
         def render_page(page, level=0, sec_prefix=[]):
@@ -302,10 +314,12 @@ class Dashboard:
                 sec_prefix[level] += 1
                 sec_prefix = sec_prefix[:level+1]
             section_num = ".".join(str(n) for n in sec_prefix)
-            outline_entries.append((f"{section_num} {page.title}", level, section_num))
+            section_title = f"{section_num} {page.title}"
+            outline_entries.append((section_title, level, section_num))
             style = styles['Heading1'] if level == 0 else styles['Heading2']
             bookmark_name = f"section_{section_num.replace('.', '_')}"
-            para = Paragraph(f'<a name="{bookmark_name}"/>{section_num} {page.title}', style)
+            para = Paragraph(f'<a name="{bookmark_name}"/>{section_title}', style)
+            heading_paragraphs.append(para)
             story.append(para)
             story.append(Spacer(1, 12))
 
@@ -325,7 +339,7 @@ class Dashboard:
                             data = [df.columns.tolist()] + df.values.tolist()
                             t = Table(data, repeatRows=1)
                             t.setStyle(TableStyle([
-                                ('BACKGROUND', (0, 0), (-1, 0), colors.HexColor("#222C36")),  # dark header
+                                ('BACKGROUND', (0, 0), (-1, 0), colors.HexColor("#222C36")),
                                 ('TEXTCOLOR', (0, 0), (-1, 0), colors.whitesmoke),
                                 ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
                                 ('FONTNAME', (0, 0), (-1, 0), 'Helvetica-Bold'),
@@ -349,22 +363,15 @@ class Dashboard:
                         fig = content
                         try:
                             import plotly.graph_objects as go
-                            import plotly.io as pio
                             if not isinstance(fig, go.Figure):
                                 raise ValueError("add_plot must be called with a plotly.graph_objects.Figure")
-                            # Set larger figure size and higher scale (DPI)
-                            fig.update_layout(
-                                margin=dict(l=10, r=10, t=30, b=30),  # Tight margins
-                                width=900,  # Larger width in pixels
-                                height=540  # Larger height in pixels
-                            )
+                            fig.update_layout(margin=dict(l=10, r=10, t=30, b=30), width=900, height=540)
                             with tempfile.NamedTemporaryFile(suffix=".png", delete=False) as tmpfile:
-                                fig.write_image(tmpfile.name, width=900, height=540, scale=3)  # scale=3 for higher DPI
+                                fig.write_image(tmpfile.name, width=600, height=360, scale=2, format="png", engine="kaleido")
                                 with open(tmpfile.name, "rb") as imgf:
                                     img_bytes = imgf.read()
                                 img_buf = io.BytesIO(img_bytes)
                                 story.append(Spacer(1, 8))
-                                # Make the image take up more space in the PDF (7x4.2 inches)
                                 story.append(Image(img_buf, width=6*inch, height=3.6*inch))
                                 story.append(Spacer(1, 12))
                             os.unlink(tmpfile.name)
@@ -382,13 +389,14 @@ class Dashboard:
 
             render_elements(page.elements)
 
-            for child in getattr(page, "children", []):
-                render_page(child, level=level+1, sec_prefix=sec_prefix.copy())
+            for i, child in enumerate(getattr(page, "children", []), start=0):
+                story.append(PageBreak())
+                child_sec_prefix = sec_prefix.copy() + [i]
+                render_page(child, level=level+1, sec_prefix=child_sec_prefix)
 
             if level == 0:
                 story.append(PageBreak())
 
-        # Add title page if requested
         if include_title_page:
             story.append(Paragraph(self.title, styles['Heading1']))
             if author:
@@ -401,11 +409,13 @@ class Dashboard:
         for page in self.pages:
             render_page(page)
 
+
         doc = MyDocTemplate(
             output_path,
             pagesize=page_size,
             rightMargin=72, leftMargin=72, topMargin=72, bottomMargin=72,
-            outline_entries=outline_entries
+            outline_entries=outline_entries,
+            headings=heading_paragraphs
         )
 
-        doc.multiBuild(story)
\ No newline at end of file
+        doc.multiBuild(story)
